@@ -241,7 +241,8 @@ close_welcome_session (GdmWelcomeSession *welcome_session)
 }
 
 static void
-load_lang_config_file (const gchar *config_file, const gchar **str_array)
+load_lang_config_file (const char  *config_file,
+                       const char **str_array)
 {
         gchar         *contents = NULL;
         gchar         *p;
@@ -277,7 +278,7 @@ load_lang_config_file (const gchar *config_file, const gchar **str_array)
                 return;
         }
 
-        str_joinv = g_strjoinv ("|", str_array);
+        str_joinv = g_strjoinv ("|", (char **) str_array);
         if (str_joinv == NULL) {
                 g_warning ("Error in joined");
                 g_free (contents);
@@ -343,7 +344,8 @@ next_line:
 }
 
 static GPtrArray *
-get_welcome_environment (GdmWelcomeSession *welcome_session, gboolean start_session)
+get_welcome_environment (GdmWelcomeSession *welcome_session,
+                         gboolean           start_session)
 {
         GPtrArray     *env;
         GHashTable    *hash;
@@ -357,7 +359,8 @@ get_welcome_environment (GdmWelcomeSession *welcome_session, gboolean start_sess
         };
         int i;
 
-        load_lang_config_file (LANG_CONFIG_FILE, optional_environment);
+        load_lang_config_file (LANG_CONFIG_FILE,
+                               (const char **) optional_environment);
         env = g_ptr_array_new ();
 
         /* create a hash table of current environment, then update keys has necessary */
@@ -534,6 +537,7 @@ spawn_child_setup (SpawnChildData *data)
 {
         struct passwd *pwent;
         struct group  *grent;
+        int            res;
 
         if (data->user_name == NULL) {
                 return;
@@ -568,14 +572,22 @@ spawn_child_setup (SpawnChildData *data)
                         g_debug ("Making per-seat gconf directory %s", gconf_dir);
                         g_mkdir (gconf_dir, S_IRWXU | S_IXGRP | S_IRGRP);
                         g_chmod (gconf_dir, S_IRWXU | S_IXGRP | S_IRGRP);
-                        chown (gconf_dir, pwent->pw_uid, grent->gr_gid);
+                        res = chown (gconf_dir, pwent->pw_uid, grent->gr_gid);
+                        if (res == -1) {
+                                g_warning ("GdmWelcomeSession: Error setting owner of per-seat gconf directory: %s",
+                                           g_strerror (errno));
+                        }
                 }
                 g_free (gconf_dir);
         }
 
         g_debug ("GdmWelcomeSession: Setting up run time dir %s", data->runtime_dir);
         g_mkdir (data->runtime_dir, 0755);
-        chown (data->runtime_dir, pwent->pw_uid, pwent->pw_gid);
+        res = chown (data->runtime_dir, pwent->pw_uid, pwent->pw_gid);
+        if (res == -1) {
+                g_warning ("GdmWelcomeSession: Error setting owner of run time directory: %s",
+                           g_strerror (errno));
+        }
 
         g_debug ("GdmWelcomeSession: Changing (uid:gid) for child process to (%d:%d)",
                  pwent->pw_uid,
@@ -803,7 +815,7 @@ parse_dbus_launch_output (const char *output,
         }
 
         if (addressp != NULL) {
-                *addressp = g_strdup (g_match_info_fetch (match_info, 1));
+                *addressp = g_match_info_fetch (match_info, 1);
         }
 
         if (pidp != NULL) {
@@ -840,6 +852,8 @@ start_dbus_daemon (GdmWelcomeSession *welcome_session)
 
         env = get_welcome_environment (welcome_session, FALSE);
 
+        std_out = NULL;
+        std_err = NULL;
         error = NULL;
         res = spawn_command_line_sync_as_user (DBUS_LAUNCH_COMMAND,
                                                welcome_session->priv->user_name,
@@ -871,6 +885,8 @@ start_dbus_daemon (GdmWelcomeSession *welcome_session)
                 g_debug ("GdmWelcomeSession: Started D-Bus daemon on pid %d", welcome_session->priv->dbus_pid);
         }
  out:
+        g_free (std_out);
+        g_free (std_err);
         return res;
 }
 
@@ -1460,6 +1476,10 @@ gdm_welcome_session_finalize (GObject *object)
 
         gdm_welcome_session_stop (welcome_session);
 
+        if (welcome_session->priv->ckc != NULL) {
+                ck_connector_unref (welcome_session->priv->ckc);
+        }
+
         g_free (welcome_session->priv->command);
         g_free (welcome_session->priv->user_name);
         g_free (welcome_session->priv->group_name);
@@ -1472,6 +1492,8 @@ gdm_welcome_session_finalize (GObject *object)
         g_free (welcome_session->priv->server_address);
         g_free (welcome_session->priv->server_dbus_path);
         g_free (welcome_session->priv->server_dbus_interface);
+        g_free (welcome_session->priv->server_env_var_name);
+        g_free (welcome_session->priv->dbus_bus_address);
 
         G_OBJECT_CLASS (gdm_welcome_session_parent_class)->finalize (object);
 }
