@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2004-2005 James M. Cape <jcape@ignore-your.tv>.
+ * Copyright (C) 2007-2008 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,7 +26,7 @@
 #define __GDM_USER__ 1
 
 #include <sys/types.h>
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 G_BEGIN_DECLS
@@ -36,20 +37,25 @@ G_BEGIN_DECLS
 
 typedef struct _GdmUser GdmUser;
 
-GType                 gdm_user_get_type           (void) G_GNUC_CONST;
+GType                 gdm_user_get_type            (void) G_GNUC_CONST;
 
-uid_t                 gdm_user_get_uid            (GdmUser   *user);
-G_CONST_RETURN char  *gdm_user_get_user_name      (GdmUser   *user);
-G_CONST_RETURN char  *gdm_user_get_real_name      (GdmUser   *user);
-G_CONST_RETURN char  *gdm_user_get_home_directory (GdmUser   *user);
-G_CONST_RETURN char  *gdm_user_get_shell          (GdmUser   *user);
-guint                 gdm_user_get_num_sessions   (GdmUser   *user);
+GdmUser              *gdm_user_new_from_object_path (const char *path);
+const char           *gdm_user_get_object_path      (GdmUser *user);
 
-GdkPixbuf            *gdm_user_render_icon        (GdmUser   *user,
-                                                   gint       icon_size);
+gulong                gdm_user_get_uid             (GdmUser   *user);
+const char           *gdm_user_get_user_name       (GdmUser   *user);
+const char           *gdm_user_get_real_name       (GdmUser   *user);
+guint                 gdm_user_get_num_sessions    (GdmUser   *user);
+gboolean              gdm_user_is_logged_in        (GdmUser   *user);
+gulong                gdm_user_get_login_frequency (GdmUser   *user);
+const char           *gdm_user_get_icon_file       (GdmUser   *user);
+const char           *gdm_user_get_primary_session_id (GdmUser *user);
 
-gint                  gdm_user_collate            (GdmUser   *user1,
-                                                   GdmUser   *user2);
+GdkPixbuf            *gdm_user_render_icon         (GdmUser   *user,
+                                                    gint       icon_size);
+
+gint                  gdm_user_collate             (GdmUser   *user1,
+                                                    GdmUser   *user2);
 
 G_END_DECLS
 
